//---------------------------------------------------------------------------
// AppsMan.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Math.hpp>
#include <UtilCls.h>
#include "Lib/Moochos/Moa/Files.h"
#include "Lib/Moochos/Moa/Lists.h"
#include "AppsMan.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Execian { namespace AppsMan {
//---------------------------------------------------------------------------
namespace {
//---------------------------------------------------------------------------
const AnsiString EntryDir = ExtractFilePath(Application->ExeName) + "Applications";
const AnsiString EntryPath = IncludeTrailingPathDelimiter(EntryDir);
const AnsiString EntryExt = ".lnk";
//---------------------------------------------------------------------------
} // End of namespace
//---------------------------------------------------------------------------
void __fastcall TAppsMan::Initialize()
{
	try {
		EnsureEntryDirExists();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TAppsMan::Uninitialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TAppsMan::Clear()
{
	TAutoStringsUpdate autoStringsUpdate(Control->Items);
	for(int i = 0; i < Control->Items->Count; i++) {
		delete Control->Items->Objects[i];
	}
	Control->Items->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TAppsMan::LoadItems(TList* AItems)
{
	TPtr<TObjectList> items = new TObjectList();
	for(TFileInfoEnumerator fileInfoEnumerator(EntryPath + "*" + EntryExt); fileInfoEnumerator.HasNext();) {
		WIN32_FIND_DATA fileInfo = fileInfoEnumerator.GetNext();
		if(lstrcmp(fileInfo.cFileName, ".") == 0 || lstrcmp(fileInfo.cFileName, "..") == 0) {
			continue;
		}
		items->Add(new TExecApp(EntryPath + fileInfo.cFileName));
	}
	AItems->Assign(items);
	items->OwnsObjects = false;
}
//---------------------------------------------------------------------------
void __fastcall TAppsMan::Execute(const AnsiString& AVerb, const AnsiString& AFileName)
{
	SHELLEXECUTEINFO executeInfo = {0};
	executeInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	executeInfo.fMask = SEE_MASK_FLAG_NO_UI;
	if(lstrcmpi(AVerb.c_str(), "properties") == 0) {
		executeInfo.fMask |= SEE_MASK_INVOKEIDLIST;
	}
	executeInfo.hwnd = Application->Handle;
	executeInfo.lpVerb = AVerb.c_str();
	executeInfo.lpFile = AFileName.c_str();
	executeInfo.nShow = SW_SHOWNORMAL;
	if(!ShellExecuteEx(&executeInfo)) {
		throw EAppsManError(AnsiString().sprintf("The program cannot execute the file \"%s\".", AFileName.c_str()));
	}
}
//---------------------------------------------------------------------------
void __fastcall TAppsMan::EnsureEntryDirExists()
{
	if(!ForceDirectories(EntryDir)) {
		throw EAppsManError("The program cannot create the Applications directory.");
	}
}
//---------------------------------------------------------------------------
TExecApp* __fastcall TAppsMan::GetItem(int AIndex)
{
	return static_cast<TExecApp*>(Control->Items->Objects[AIndex]);
}
//---------------------------------------------------------------------------
int __fastcall TAppsMan::GetCount()
{
	return Control->Items->Count;
}
//---------------------------------------------------------------------------
__fastcall TAppsMan::TAppsMan(TComboBox* AControl) :
	TObject(),
	FControl(AControl)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TAppsMan::~TAppsMan()
{
	Clear();
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TAppsMan::Organize()
{
	Execute("open", EntryDir);
}
//---------------------------------------------------------------------------
void __fastcall TAppsMan::Update()
{
	TPtr<TObjectList> items = new TObjectList();
	LoadItems(items);
	Clear();
	Control->Items->Capacity = items->Count;
	for(int i = 0; i < items->Count; i++) {
		Control->Items->AddObject(static_cast<TExecApp*>(items->Items[i])->Title, items->Items[i]);
		items->OwnsObjects = false;
		items->Items[i] = NULL;
		items->OwnsObjects = true;
	}
	Control->ItemIndex = EnsureRange(0, -1, Control->Items->Count - 1);
}
//---------------------------------------------------------------------------
void __fastcall TAppsMan::Open()
{
	Execute("open", Items[Control->ItemIndex]->FileName);
}
//---------------------------------------------------------------------------
void __fastcall TAppsMan::OpenDirectory()
{
	Execute("open", ExtractFileDir(ExpandFileName(Items[Control->ItemIndex]->FileName)));
}
//---------------------------------------------------------------------------
void __fastcall TAppsMan::ShowProperties()
{
	Execute("properties", EntryPath + Control->Text + EntryExt);
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Execian::AppsMan
//---------------------------------------------------------------------------

