//---------------------------------------------------------------------------
// DocsMan.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Clipbrd.hpp>
#include <UtilCls.h>
#include "Lib/Moochos/Moa/Files.h"
#include "Lib/Moochos/Moa/ListControls.h"
#include "Lib/Moochos/Moa/Lists.h"
#include "DocsMan.h"
#include "ResultsDialogUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Execian { namespace DocsMan {
//---------------------------------------------------------------------------
namespace {
//---------------------------------------------------------------------------
const AnsiString DefExportFileName = "Execian Documents.txt";
//---------------------------------------------------------------------------
} // End of namespace
//---------------------------------------------------------------------------
void __fastcall TDocsMan::Initialize()
{
	try {
		FItemList = new TObjectList();

		FAddFilesDialog = new TOpenDialog(NULL);
		AddFilesDialog->Title = "Add Files (Including shortcut files)";
		AddFilesDialog->Filter = "All|*";
		AddFilesDialog->Options = AddFilesDialog->Options << ofAllowMultiSelect << ofNoDereferenceLinks;

		FImportDialog = new TOpenDialog(NULL);
		ImportDialog->Title = "Import";
		ImportDialog->Filter = "Text (*.txt)|*.txt|All|*";

		FExportDialog = new TSaveDialog(NULL);
		ExportDialog->Title = "Export";
		ExportDialog->DefaultExt = "txt";
		ExportDialog->Filter = "Text (*.txt)|*.txt|All|*";
		ExportDialog->Options = FExportDialog->Options << ofOverwritePrompt;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::Uninitialize()
{
	delete FExportDialog;
	FExportDialog = NULL;

	delete FImportDialog;
	FImportDialog = NULL;

	delete FAddFilesDialog;
	FAddFilesDialog = NULL;

	delete FItemList;
	FItemList = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::AddFiles(TStrings* AFiles)
{
	TPtr<TObjectList> items = new TObjectList();
	items->Capacity = AFiles->Count;
	for(int i = 0; i < AFiles->Count; i++) {
		items->Add(new TExecDoc(QualifyFileName(AFiles->Strings[i])));
	}
	AddItems(items);
	items->OwnsObjects = false;
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::AddItems(TList* AItems)
{
	TAutoListItemsUpdator listItemsUpdator(Control->Items);
	int oldCount = Count;
	int newCount = Count + AItems->Count;
	try {
		ItemList->Capacity = newCount;
		for(int i = 0; i < AItems->Count; i++) {
			ItemList->Add(static_cast<TObject*>(AItems->Items[i]));
		}

		TAutoListViewOnData listViewOnData(Control);
		Control->Items->Count = newCount;
		for(int i = 0; i < newCount - oldCount; i++) {
			Control->Items->Item[oldCount + i]->Selected = true;
		}
		Control->Items->Item[newCount - 1]->Focused = true;
		Control->Items->Item[newCount - 1]->MakeVisible(true);
		Control->Invalidate();
	}
	catch(...) {
		ItemList->OwnsObjects = false;
		ItemList->Count = oldCount;
		ItemList->OwnsObjects = true;
		Control->Items->Count = oldCount;
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::ReorderItems(const TIntegerDynArray& AOrder)
{
	TPtr<TObjectList> newItemList = new TObjectList(false);
	{
		newItemList->Count = ItemList->Count;
		for(int i = 0; i < ItemList->Count; i++) {
			newItemList->Items[i] = ItemList->Items[AOrder[i]];
		}
	}
	TAutoObjectListOwnsObjects autoOwnsObjects(ItemList, false);
	ItemList->Assign(newItemList);
}
//---------------------------------------------------------------------------
TExecDoc* __fastcall TDocsMan::GetItem(int AIndex)
{
	return static_cast<TExecDoc*>(ItemList->Items[AIndex]);
}
//---------------------------------------------------------------------------
int __fastcall TDocsMan::GetCount()
{
	return ItemList->Count;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocsMan::QualifyFileName(const AnsiString& AFileName) const
{
	if(IsFileNameMask(AFileName)) {
		throw EDocsManError(AnsiString().sprintf(
			"The program cannot accept the file name \"%s\", because it contains wildcard characters.",
			AFileName.c_str()));
	}
	if(IsRelativeFileName(AFileName)) {
		throw EDocsManError(AnsiString().sprintf(
			"The program cannot accept the file name \"%s\", because it is in a relative form.",
			AFileName.c_str()));
	}
	AnsiString newFileName = ExtractLongPathName(ExpandFileName(AFileName));
	if(newFileName.Length() == 0) {
		return AFileName;
	}
	return newFileName;
}
//---------------------------------------------------------------------------
int __fastcall TDocsMan::CountElements(const TBooleanDynArray& AArray, bool AValue) const
{
	int count = 0;
	for(int i = 0; i < AArray.Length; i++) {
		if((AArray[i] ? 1 : 0) != (AValue ? 1 : 0)) {
			continue;
		}
		count++;
	}
	return count;
}
//---------------------------------------------------------------------------
__fastcall TDocsMan::TDocsMan(TListView* AControl) :
	TObject(),
	FControl(AControl),
	FItemList(NULL),
	FAddFilesDialog(NULL),
	ImportDialog(NULL),
	ExportDialog(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TDocsMan::~TDocsMan()
{
	Clear(false);
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::AddFiles()
{
	AddFilesDialog->FileName = AnsiString();
	if(!AddFilesDialog->Execute()) {
		Abort();
	}
	AddFiles(AddFilesDialog->Files);
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::AddDirectory()
{
	AnsiString directory = GetCurrentDir();
	if(!SelectDirectory("Choose a directory to add.", WideString(), directory)) {
		Abort();
	}
	SetCurrentDir(directory);
	TPtr<TStrings> files = new TStringList();
	files->Add(directory);
	AddFiles(files);
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::AddDroppedFiles(TStrings* AFiles)
{
	AddFiles(AFiles);
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::RemoveSelected(bool AConfirm)
{
	TBooleanDynArray canRemove;
	canRemove.Length = Count;
	for(int i = 0; i < Count; i++) {
		canRemove[i] = Control->Items->Item[i]->Selected;
	}
	Remove(canRemove, AConfirm);
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::RemoveByResult(TExecDocResult AResult, bool AConfirm)
{
	TBooleanDynArray canRemove;
	canRemove.Length = Count;
	for(int i = 0; i < Count; i++) {
		canRemove[i] = Items[i]->Result == AResult;
	}
	Remove(canRemove, AConfirm);
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::Remove(const TBooleanDynArray& ACanRemove, bool AConfirm)
{
	if(AConfirm) {
		if(Application->MessageBox(
			AnsiString().sprintf("Remove %d items?", CountElements(ACanRemove, true)).c_str(),
			Application->Title.c_str(),
			MB_YESNO | MB_ICONWARNING | MB_SETFOREGROUND) != IDYES) {
			Abort();
		}
	}

	TAutoListViewOnData listViewOnData(Control);
	TPtr<TListControlShell> controlShell = new TListControlShell(new TListViewShellView(Control));
	controlShell->Update();
	controlShell->DeleteItems(ACanRemove);
	controlShell->UpdateControl();

	for(int i = 0; i < ItemList->Count; i++) {
		if(!ACanRemove[i]) {
			continue;
		}
		ItemList->Items[i] = NULL;
	}
	ItemList->Pack();
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::Clear(bool AConfirm)
{
	if(AConfirm) {
		if(Application->MessageBox(
			"Clear the list?",
			Application->Title.c_str(),
			MB_YESNO | MB_ICONWARNING | MB_SETFOREGROUND) != IDYES) {
			Abort();
		}
	}
	Control->Items->BeginUpdate();
	try {
		TAutoListViewOnData listViewOnData(Control);
		Control->Items->Count = 0;
		ItemList->Clear();
	}
	__finally {
		Control->Items->EndUpdate();
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::Import()
{
	try {
		ImportDialog->FileName = AnsiString();
		if(!ImportDialog->Execute()) {
			Abort();
		}
		TPtr<TStrings> list = new TStringList();
		list->LoadFromFile(ImportDialog->FileName);
		PackByString(list, list);
		AddFiles(list);
	}
	catch(EStreamError& e) {
		throw EDocsManError("The program cannot load the document list.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::Export()
{
	try {
		ExportDialog->FileName = DefExportFileName;
		if(!ExportDialog->Execute()) {
			Abort();
		}
		TPtr<TStrings> list = new TStringList();
		for(int i = 0; i < Count; i++) {
			if(!Control->Items->Item[i]->Selected) {
				continue;
			}
			list->Add(Items[i]->FileName);
		}
		list->SaveToFile(ExportDialog->FileName);
	}
	catch(EStreamError& e) {
		throw EDocsManError("The program cannot save the document list.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::Cut()
{
	Copy();
	RemoveSelected(false);
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::Copy()
{
	TPtr<TStrings> fileNames = new TStringList();
	fileNames->Capacity = Control->SelCount;
	for(int i = 0; i < Count; i++) {
		if(!Control->Items->Item[i]->Selected) {
			continue;
		}
		fileNames->Add(Items[i]->FileName);
	}
	Clipboard()->AsText = fileNames->Text;
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::Paste()
{
	TPtr<TStrings> fileNames = new TStringList();
	fileNames->Text = Clipboard()->AsText;
	PackByString(fileNames, fileNames);
	AddFiles(fileNames);
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::SelectAll()
{
	TAutoListViewOnData listViewOnData(Control);
	TPtr<TListControlShell> controlShell = new TListControlShell(new TListViewShellView(Control));
	controlShell->Update();
	controlShell->SelectAll();
	controlShell->UpdateControl();
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::SelectByResult(TExecDocResult AResult)
{
	TAutoListViewOnData listViewOnData(Control);
	TPtr<TListControlShell> controlShell = new TListControlShell(new TListViewShellView(Control));
	controlShell->Update();
	for(int i = 0; i < Count; i++) {
		controlShell->States[i] = Items[i]->Result == AResult ?
			(controlShell->States[i] | lsSelected) :
			(controlShell->States[i] & ~lsSelected);
	}
	controlShell->UpdateControl();
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::ReverseSelection()
{
	TAutoListViewOnData listViewOnData(Control);
	TPtr<TListControlShell> controlShell = new TListControlShell(new TListViewShellView(Control));
	controlShell->Update();
	controlShell->ReverseSelection();
	controlShell->UpdateControl();
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::MoveUp()
{
	TAutoListViewOnData listViewOnData(Control);
	TPtr<TListControlShell> controlShell = new TListControlShell(new TListViewShellView(Control));
	controlShell->Update();
	ReorderItems(controlShell->MoveSelected(-1));
	controlShell->UpdateControl();
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::MoveDown()
{
	TAutoListViewOnData listViewOnData(Control);
	TPtr<TListControlShell> controlShell = new TListControlShell(new TListViewShellView(Control));
	controlShell->Update();
	ReorderItems(controlShell->MoveSelected(1));
	controlShell->UpdateControl();
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::SortByName()
{
	struct Local
	{
		static int __fastcall Compare(int AIndex1, int AIndex2, void* AData)
		{
			int difference = AnsiCompareStr(
				static_cast<TDocsMan*>(AData)->Items[AIndex1]->Name,
				static_cast<TDocsMan*>(AData)->Items[AIndex2]->Name);
			if(difference == 0) {
				difference = AIndex1 - AIndex2;
			}
			return difference;
		}
	};

	TAutoListViewOnData listViewOnData(Control);
	TPtr<TListControlShell> controlShell = new TListControlShell(new TListViewShellView(Control));
	controlShell->Update();
	ReorderItems(controlShell->Sort(Local::Compare, this));
	controlShell->UpdateControl();
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::SortByDirectory()
{
	struct Local
	{
		static int __fastcall Compare(int AIndex1, int AIndex2, void* AData)
		{
			int difference = AnsiCompareStr(
				static_cast<TDocsMan*>(AData)->Items[AIndex1]->Directory,
				static_cast<TDocsMan*>(AData)->Items[AIndex2]->Directory);
			if(difference == 0) {
				difference = AIndex1 - AIndex2;
			}
			return difference;
		}
	};

	TAutoListViewOnData listViewOnData(Control);
	TPtr<TListControlShell> controlShell = new TListControlShell(new TListViewShellView(Control));
	controlShell->Update();
	ReorderItems(controlShell->Sort(Local::Compare, this));
	controlShell->UpdateControl();
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::SortByDirAndName()
{
	struct Local
	{
		static int __fastcall Compare(int AIndex1, int AIndex2, void* AData)
		{
			int difference = AnsiCompareStr(
				static_cast<TDocsMan*>(AData)->Items[AIndex1]->FileName,
				static_cast<TDocsMan*>(AData)->Items[AIndex2]->FileName);
			if(difference == 0) {
				difference = AIndex1 - AIndex2;
			}
			return difference;
		}
	};

	TAutoListViewOnData listViewOnData(Control);
	TPtr<TListControlShell> controlShell = new TListControlShell(new TListViewShellView(Control));
	controlShell->Update();
	ReorderItems(controlShell->Sort(Local::Compare, this));
	controlShell->UpdateControl();
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::SortByResult()
{
	struct Local
	{
		static int __fastcall GetPriority(TExecDocResult AResult)
		{
			switch(AResult) {
			case drSuccess:
				return 0;
			case drFailure:
				return 1;
			case drNoResult:
				return 2;
			default:
				throw EDocsManError("An invalid identifier is specified.");
			}
		}

		static int __fastcall Compare(int AIndex1, int AIndex2, void* AData)
		{
			int difference =
				GetPriority(static_cast<TDocsMan*>(AData)->Items[AIndex1]->Result) -
				GetPriority(static_cast<TDocsMan*>(AData)->Items[AIndex2]->Result);
			if(difference == 0) {
				difference = AIndex1 - AIndex2;
			}
			return difference;
		}
	};

	TAutoListViewOnData listViewOnData(Control);
	TPtr<TListControlShell> controlShell = new TListControlShell(new TListViewShellView(Control));
	controlShell->Update();
	ReorderItems(controlShell->Sort(Local::Compare, this));
	controlShell->UpdateControl();
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::SortByError()
{
	struct Local
	{
		static int __fastcall GetPriority(TExecDocError AError)
		{
			switch(AError) {
			case deFileNotFound:
				return 0;
			case deFileNameNotFound:
				return 1;
			case deDdeFailed:
				return 2;
			case deNoAccociation:
				return 3;
			case deAccessDenied:
				return 4;
			case deDllNotFound:
				return 5;
			case deCanceled:
				return 6;
			case deNotEnoughMemory:
				return 7;
			case deSharingViolation:
				return 8;
			case deUnknown:
				return 9;
			case deNoError:
				return 10;
			default:
				throw EDocsManError("An invalid identifier is specified.");
			}
		}

		static int __fastcall Compare(int AIndex1, int AIndex2, void* AData)
		{
			int difference =
				GetPriority(static_cast<TDocsMan*>(AData)->Items[AIndex1]->Error) -
				GetPriority(static_cast<TDocsMan*>(AData)->Items[AIndex2]->Error);
			if(difference == 0) {
				difference = AIndex1 - AIndex2;
			}
			return difference;
		}
	};

	TAutoListViewOnData listViewOnData(Control);
	TPtr<TListControlShell> controlShell = new TListControlShell(new TListViewShellView(Control));
	controlShell->Update();
	ReorderItems(controlShell->Sort(Local::Compare, this));
	controlShell->UpdateControl();
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::Invert()
{
	TAutoListViewOnData listViewOnData(Control);
	TPtr<TListControlShell> controlShell = new TListControlShell(new TListViewShellView(Control));
	controlShell->Update();
	ReorderItems(controlShell->Invert());
	controlShell->UpdateControl();
}
//---------------------------------------------------------------------------
void __fastcall TDocsMan::ShowResults()
{
	TPtr<TResultsDialog> dialog = new TResultsDialog(NULL);
	dialog->NoResult = CountResults(drNoResult);
	dialog->Success = CountResults(drSuccess);
	dialog->Failure = CountResults(drFailure);
	dialog->Execute();
}
//---------------------------------------------------------------------------
int __fastcall TDocsMan::CountResults(TExecDocResult AResult)
{
	int count = 0;
	for(int i = 0; i < Count; i++) {
		if(Items[i]->Result != AResult) {
			continue;
		}
		count++;
	}
	return count;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Execian::DocsMan
//---------------------------------------------------------------------------

