//---------------------------------------------------------------------------
// ExecTask.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "Lib/Moochos/Moa/Files.h"
#include "ExecTask.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Execian { namespace ExecTask {
//---------------------------------------------------------------------------
void __fastcall TExecTask::Initialize()
{
	try {
		if(App != NULL) {
			FCurApp = new TExecApp();
		}
		FCurDoc = new TExecDoc();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::Uninitialize()
{
	delete FCurDoc;
	FCurDoc = NULL;

	delete FCurApp;
	FCurApp = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::ResetDocs()
{
	for(int i = 0; i < All; i++) {
		Docs[i]->Result = drNoResult;
		Docs[i]->Error = deNoError;
	}
	DoAllReset();
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::UpdateCurFiles()
{
	Synchronize(UpdateCurFilesInMainThread, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::UpdateCurFilesInMainThread(void* AData)
{
	if(App != NULL) {
		FCurApp->Assign(App);
	}
	FCurDoc->Assign(Docs[Current]);
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::UpdateOriFiles()
{
	Synchronize(UpdateOriFilesInMainThread, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::UpdateOriFilesInMainThread(void* AData)
{
	Docs[Current]->Result = FCurDoc->Result;
	Docs[Current]->Error = FCurDoc->Error;
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::EnsureNotPaused()
{
	while(Paused) {
		EnsureNotCanceled();
		Sleep(100);
		Idle();
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::EnsureNotCanceled()
{
	if(Canceled) {
		throw EExecTaskCancel("The task is canceled.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::CommandExecute(TObject* ASender)
{
	DoOneExecute();
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::CommandResult(TObject* ASender)
{
	UpdateOriFiles();
	DoOneResult();
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::CommandWait(TObject* ASender)
{
	FSkippedWait = false;
	DoOneWait();
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::CommandCanWaitMore(TObject* ASender, bool& ACanWaitMore)
{
	Idle();
	ACanWaitMore = !Canceled && !SkippedWait;
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoSkipWait()
{
	Synchronize(DoSkipWaitInMainThread, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoSkipWaitInMainThread(void* AData)
{
	if(OnSkipWait != NULL) {
		OnSkipWait(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoPause()
{
	Synchronize(DoPauseInMainThread, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoPauseInMainThread(void* AData)
{
	if(OnPause != NULL) {
		OnPause(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoResume()
{
	Synchronize(DoResumeInMainThread, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoResumeInMainThread(void* AData)
{
	if(OnResume != NULL) {
		OnResume(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoAllReset()
{
	Synchronize(DoAllResetInMainThread, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoAllResetInMainThread(void* AData)
{
	if(OnAllReset != NULL) {
		OnAllReset(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoOneExecute()
{
	Synchronize(DoOneExecuteInMainThread, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoOneExecuteInMainThread(void* AData)
{
	if(OnOneExecute != NULL) {
		OnOneExecute(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoOneResult()
{
	Synchronize(DoOneResultInMainThread, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoOneResultInMainThread(void* AData)
{
	if(OnOneResult != NULL) {
		OnOneResult(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoOneWait()
{
	Synchronize(DoOneWaitInMainThread, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoOneWaitInMainThread(void* AData)
{
	if(OnOneWait != NULL) {
		OnOneWait(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoOneWaitFail()
{
	Synchronize(DoOneWaitFailInMainThread, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::DoOneWaitFailInMainThread(void* AData)
{
	if(OnOneWaitFail != NULL) {
		OnOneWaitFail(this);
	}
}
//---------------------------------------------------------------------------
TExecDoc* __fastcall TExecTask::GetDoc(int AIndex)
{
	return FDocs[AIndex];
}
//---------------------------------------------------------------------------
int __fastcall TExecTask::GetAll()
{
	return FDocs.Length;
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::Main()
{
	try {
		ResetDocs();
		while(Current < All) {
			try {
				EnsureNotCanceled();
				EnsureNotPaused();
				UpdateCurFiles();
				TPtr<TExecCommand> command = new TExecCommand(FCurApp, FCurDoc);
				command->OnExecute = CommandExecute;
				command->OnResult = CommandResult;
				command->OnWait = CommandWait;
				command->OnCanWaitMore = CommandCanWaitMore;
				command->Execute();
			}
			catch(EExecExecuteError& e) {
				// Do nothing
			}
			catch(EExecWaitError& e) {
				DoOneWaitFail();
			}
			FCurrent++;
			SetProgress(Current == 0 ? 0 : (Current * 100i64 / All));
			Idle();
		}
	}
	catch(EExecTaskCancel& e) {
		// Do nothing
	}
}
//---------------------------------------------------------------------------
__fastcall TExecTask::TExecTask(TExecApp* AApp, DynamicArray<TExecDoc*> ADocs) :
	TTask(true),
	FApp(AApp),
	FDocs(ADocs.Copy()),
	FCurApp(NULL),
	FCurDoc(NULL),
	FCurrent(0),
	FSkippedWait(false),
	FPaused(false),
	FOnSkipWait(NULL),
	FOnPause(NULL),
	FOnResume(NULL),
	FOnAllReset(NULL),
	FOnOneExecute(NULL),
	FOnOneResult(NULL),
	FOnOneWait(NULL),
	FOnOneWaitFail(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TExecTask::~TExecTask()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::SkipWait()
{
	if(FSkippedWait) {
		return;
	}
	FSkippedWait = true;
	DoSkipWait();
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::Pause()
{
	if(Paused || Canceled || !IsBusy()) {
		return;
	}
	FPaused = true;
	DoPause();
}
//---------------------------------------------------------------------------
void __fastcall TExecTask::Resume()
{
	if(!Paused || Canceled || !IsBusy()) {
		return;
	}
	FPaused = false;
	DoResume();
}
//---------------------------------------------------------------------------
void __fastcall TExecCommand::Main()
{
	MainWait(MainExecute());
}
//---------------------------------------------------------------------------
HANDLE __fastcall TExecCommand::MainExecute()
{
	HANDLE processHandle = NULL;
	try {
		DoExecute();
		SHELLEXECUTEINFO executeInfo = {0};
		AnsiString file;
		AnsiString parameters;
		AnsiString directory = FileExists(file) ? ExtractFileDir(ExpandFileName(file)) : file;
		int showCommand;
		if(App != NULL && !IsAbsoluteFileName(App->FileName)) {
			throw EExecCommandError("The file name of the application is invalid.");
		}
		if(!IsAbsoluteFileName(Doc->FileName)) {
			throw EExecCommandError("The file name of the document is invalid.");
		}
		if(App == NULL) {
			file = Doc->FileName;
			parameters = AnsiString();
			directory = ExtractFileDir(file);
			showCommand = SW_SHOWNORMAL;
		}
		else {
			file = App->FileName;
			parameters = AnsiString('\"') + Doc->FileName + AnsiString('\"');
			directory = App->WorkingDir;
			showCommand = App->WindowState;
		}
		executeInfo.cbSize = sizeof(SHELLEXECUTEINFO);
		executeInfo.fMask = SEE_MASK_FLAG_NO_UI | SEE_MASK_NOCLOSEPROCESS;
		executeInfo.hwnd = NULL;
		executeInfo.lpVerb = NULL;
		executeInfo.lpFile = file.c_str();
		executeInfo.lpParameters = static_cast<LPCSTR>(parameters.data());
		executeInfo.lpDirectory = static_cast<LPCSTR>(directory.data());
		executeInfo.nShow = showCommand;
		::SetLastError(NO_ERROR);
		if(!ShellExecuteEx(&executeInfo)) {
			Doc->Error = OsErrorToExecDocError(::GetLastError());
			Doc->Result = drFailure;
			DoResult();
			throw EExecExecuteError("The program cannot exexute the document.");
		}
		processHandle = executeInfo.hProcess;
		Doc->Result = drSuccess;
		Doc->Error = deNoError;
		DoResult();
	}
	catch(...) {
		if(processHandle != NULL) {
			CloseHandle(processHandle);
		}
		throw;
	}
	return processHandle;
}
//---------------------------------------------------------------------------
void __fastcall TExecCommand::MainWait(HANDLE AProcess)
{
	DoWait();
	if(AProcess == NULL) {
		throw EExecWaitError("The program cannot detact when the application terminates.");
	}
	try {
		while(DoCanWaitMore()) {
			DWORD result = ::WaitForSingleObject(AProcess, WaitInterval);
			if(result == WAIT_ABANDONED || result == WAIT_OBJECT_0) {
				break;
			}
			else if(result == WAIT_TIMEOUT) {
				// Do nothing
			}
			else if(result == WAIT_FAILED) {
				throw EExecWaitError("The program cannot wait until the application terminates.");
			}
		}
	}
	__finally {
		::CloseHandle(AProcess);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecCommand::DoExecute()
{
	if(OnExecute != NULL) {
		OnExecute(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecCommand::DoResult()
{
	if(OnResult != NULL) {
		OnResult(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecCommand::DoWait()
{
	if(OnWait != NULL) {
		OnWait(this);
	}
}
//---------------------------------------------------------------------------
bool __fastcall TExecCommand::DoCanWaitMore()
{
	bool canWaitMore = true;
	if(OnCanWaitMore != NULL) {
		OnCanWaitMore(this, canWaitMore);
	}
	return canWaitMore;
}
//---------------------------------------------------------------------------
TExecDocError __fastcall TExecCommand::OsErrorToExecDocError(DWORD AOsError)
{
	switch(::GetLastError()) {
	case NO_ERROR:
		return deNoError;
	case ERROR_FILE_NOT_FOUND:
		return deFileNotFound;
	case ERROR_PATH_NOT_FOUND:
		return deFileNameNotFound;
	case ERROR_DDE_FAIL:
		return deDdeFailed;
	case ERROR_NO_ASSOCIATION:
		return deNoAccociation;
	case ERROR_ACCESS_DENIED:
		return deAccessDenied;
	case ERROR_DLL_NOT_FOUND:
		return deDllNotFound;
	case ERROR_CANCELLED:
		return deCanceled;
	case ERROR_NOT_ENOUGH_MEMORY:
		return deNotEnoughMemory;
	case ERROR_SHARING_VIOLATION:
		return deSharingViolation;
	default:
		return deUnknown;
	}
}
//---------------------------------------------------------------------------
__fastcall TExecCommand::TExecCommand(TExecApp* AApp, TExecDoc* ADoc) :
	TObject(),
	FApp(AApp),
	FDoc(ADoc),
	FWaitInterval(100),
	FOnExecute(NULL),
	FOnResult(NULL),
	FOnWait(NULL),
	FOnCanWaitMore(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TExecCommand::~TExecCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TExecCommand::Execute()
{
	Main();
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Execian::ExecTask
//---------------------------------------------------------------------------

