//---------------------------------------------------------------------------
// ExecutionMan.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "Lib/Moochos/Moa/ListControls.h"
#include "ExecTask.h"
#include "ExecutionMan.h"
#include "ProgressDialogUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Execian { namespace ExecutionMan {
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::HandleMessages()
{
	if(Task->Async) {
		return;
	}
	Application->ProcessMessages();
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::HandleCanceling()
{
	if(!Canceling) {
		return;
	}
	FCanceling = false;
	if(Application->MessageBox(
		"Cancel the task?",
		Application->Title.c_str(),
		MB_YESNO | MB_ICONWARNING | MB_SETFOREGROUND) != IDYES) {
		return;
	}
	Task->Cancel();
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::HandleResults()
{
	if(ProgressDialog->Failure == 0) {
		return;
	}
	Application->MessageBox(
		AnsiString().sprintf("There are %d failure documents.", ProgressDialog->Failure).c_str(),
		Application->Title.c_str(),
		MB_OK | MB_ICONWARNING | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskStart(TObject* ASender)
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskEnd(TObject* ASender)
{
	ProgressDialog->Application = AnsiString();
	ProgressDialog->Document = AnsiString();
	ProgressDialog->Status = ProgressDialog->IsCompleted() ? "Done" : "Canceled";
	ProgressDialog->Progress = 0;
	ProgressDialog->Finish();
	ProgressDialog->ForceClose();
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskCancel(TObject* ASender)
{
	if(Task->Canceled) {
		return;
	}
	FCanceling = false;
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskProgress(TObject* ASender)
{
	ProgressDialog->Progress = Task->Progress;
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskIdle(TObject* ASender)
{
	HandleMessages();
	HandleCanceling();
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskException(TObject* ASender, Exception* AException)
{
	Application->ShowException(AException);
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskSkipWait(TObject* ASender)
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskPause(TObject* ASender)
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskResume(TObject* ASender)
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskAllReset(TObject* ASender)
{
	ProgressDialog->NoResult = Task->All;
	ProgressDialog->Success = 0;
	ProgressDialog->Failure = 0;
	DocsControl->Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskOneExecute(TObject* ASender)
{
	ProgressDialog->Application = Task->App == NULL ? AnsiString("(Default)") : Task->App->Title;
	ProgressDialog->Document = Task->Docs[Task->Current]->FileName;
	ProgressDialog->Status = "Executing...";
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskOneResult(TObject* ASender)
{
	ProgressDialog->NoResult = ProgressDialog->NoResult - 1;
	TExecDoc* doc = Task->Docs[Task->Current];
	ProgressDialog->ResultValues[doc->Result] = ProgressDialog->ResultValues[doc->Result] + 1;
	DocsControl->UpdateItems(Task->Current, Task->Current);
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskOneWait(TObject* ASender)
{
	ProgressDialog->Status = "Waiting...";
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::TaskOneWaitFail(TObject* ASender)
{
	Task->Pause();
	ProgressDialog->Status = "Wait failed";
	ProgressDialog->Pause();
	Application->MessageBox(
		"The program cannot detect the termination of the process.\r\n\r\nNow the task is paused.",
		Application->Title.c_str(),
		MB_OK | MB_ICONWARNING | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::ProgressDialogStart(TObject* ASender)
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::ProgressDialogEnd(TObject* ASender)
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::ProgressDialogExecute(TObject* ASender)
{
	Task->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::ProgressDialogUnmark(TObject* ASender)
{
	TExecDoc* doc = Task->Docs[Task->Current];
	if(doc->Result == drNoResult) {
		return;
	}
	ProgressDialog->ResultValues[doc->Result] = ProgressDialog->ResultValues[doc->Result] - 1;
	doc->Result = drNoResult;
	ProgressDialog->NoResult = ProgressDialog->NoResult + 1;
	DocsControl->UpdateItems(Task->Current, Task->Current);
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::ProgressDialogSkipWait(TObject* ASender)
{
	Task->SkipWait();
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::ProgressDialogPause(TObject* ASender)
{
	Task->Pause();
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::ProgressDialogResume(TObject* ASender)
{
	Task->Resume();
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::ProgressDialogCancel(TObject* ASender)
{
	if(Task->Canceled || Canceling) {
		return;
	}
	FCanceling = true;
}
//---------------------------------------------------------------------------
__fastcall TExecutionMan::TExecutionMan(TListView* ADocsControl) :
	TObject(),
	FTask(NULL),
	FCanceling(false),
	FDocsControl(ADocsControl)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TExecutionMan::~TExecutionMan()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TExecutionMan::Execute(TExecApp* AApp, DynamicArray<TExecDoc*> ADocs)
{
	TPtr<TExecTask> task = new TExecTask(AApp, ADocs);
	FTask = task;
	Task->OnStart = TaskStart;
	Task->OnEnd = TaskEnd;
	Task->OnCancel = TaskCancel;
	Task->OnProgress = TaskProgress;
	Task->OnIdle = TaskIdle;
	Task->OnException = TaskException;
	Task->OnSkipWait = TaskSkipWait;
	Task->OnPause = TaskPause;
	Task->OnResume = TaskResume;
	Task->OnAllReset = TaskAllReset;
	Task->OnOneExecute = TaskOneExecute;
	Task->OnOneResult = TaskOneResult;
	Task->OnOneWait = TaskOneWait;
	Task->OnOneWaitFail = TaskOneWaitFail;
	FCanceling = false;
	ProgressDialog->Application = AnsiString();
	ProgressDialog->Document = AnsiString();
	ProgressDialog->Status = AnsiString();
	ProgressDialog->Progress = 0;
	ProgressDialog->NoResult = 0;
	ProgressDialog->Success = 0;
	ProgressDialog->Failure = 0;
	ProgressDialog->OnStart = ProgressDialogStart;
	ProgressDialog->OnEnd = ProgressDialogEnd;
	ProgressDialog->OnExecute = ProgressDialogExecute;
	ProgressDialog->OnUnmark = ProgressDialogUnmark;
	ProgressDialog->OnSkipWait = ProgressDialogSkipWait;
	ProgressDialog->OnPause = ProgressDialogPause;
	ProgressDialog->OnResume = ProgressDialogResume;
	ProgressDialog->OnCancel = ProgressDialogCancel;
	ProgressDialog->Execute();
	HandleResults();
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Execian:ExecutionMan:
using namespace Moochos::Execian::ExecutionMan;
//---------------------------------------------------------------------------

