//---------------------------------------------------------------------------
// Files.cpp
//
// Version: 1.1.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <mbstring.h>
#include "Files.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Files {
//---------------------------------------------------------------------------
void __fastcall TFileInfoEnumerator::FindFirst(const AnsiString& AFileName)
{
	HANDLE newHandle = ::FindFirstFile(AFileName.c_str(), &FData);
	if(newHandle == INVALID_HANDLE_VALUE) {
		FFound = false;
		return;
	}
	FHandle = newHandle;
	FFound = true;
}
//---------------------------------------------------------------------------
void __fastcall TFileInfoEnumerator::FindNext()
{
	if(!::FindNextFile(FHandle, &FData)) {
		FindClose();
		FFound = false;
		return;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileInfoEnumerator::FindClose()
{
	if(FHandle == NULL) {
		return;
	}
	::FindClose(FHandle);
	FHandle = NULL;
}
//---------------------------------------------------------------------------
__fastcall TFileInfoEnumerator::TFileInfoEnumerator(const AnsiString& AFileName) :
	FHandle(NULL),
	FData(),
	FFound(false)
{
	FindFirst(AFileName);
}
//---------------------------------------------------------------------------
__fastcall TFileInfoEnumerator::~TFileInfoEnumerator()
{
	FindClose();
}
//---------------------------------------------------------------------------
bool __fastcall TFileInfoEnumerator::HasNext()
{
	return FFound;
}
//---------------------------------------------------------------------------
WIN32_FIND_DATA __fastcall TFileInfoEnumerator::GetNext()
{
	if(!HasNext()) {
		throw EFileError("The program cannot find any more file informations.");
	}
	WIN32_FIND_DATA next = FData;
	FindNext();
	return next;
}
//---------------------------------------------------------------------------
AnsiString __fastcall ExtractLongPathName(const AnsiString AFileName)
{
	if(AFileName.Length() >= MAX_PATH) {
		return AnsiString();
	}
	const DWORD bufferSize = MAX_PATH;
	TCHAR buffer[bufferSize] = {0};
	DWORD didCopySize = ::GetLongPathName(AFileName.c_str(), buffer, bufferSize);
	if(didCopySize == 0 || didCopySize > bufferSize) {
		return AnsiString();
	}
	return AnsiString(buffer, didCopySize);
}
//---------------------------------------------------------------------------
bool __fastcall DriveExists(const AnsiString& AFileName)
{
	return GetDriveType(AFileName.c_str()) != 1;
}
//---------------------------------------------------------------------------
bool __fastcall FileOrDirExists(const AnsiString& AFileName)
{
	return GetFileAttributes(AFileName.c_str()) != 0xFFFFFFFF;
}
//---------------------------------------------------------------------------
bool __fastcall IsFileNameMask(const AnsiString& AFileName)
{
	return _mbspbrk(AFileName.c_str(), "*?") != NULL;
}
//---------------------------------------------------------------------------
bool __fastcall IsAbsoluteFileName(const AnsiString& AFileName)
{
	return _mbschr(AFileName.c_str(), ':') != NULL;
}
//---------------------------------------------------------------------------
bool __fastcall IsRelativeFileName(const AnsiString& AFileName)
{
	return !IsAbsoluteFileName(AFileName);
}
//---------------------------------------------------------------------------
bool __fastcall IsDirectory(const WIN32_FIND_DATA& AFindData)
{
	return (AFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
}
//---------------------------------------------------------------------------
bool __fastcall IsDotFileName(const WIN32_FIND_DATA& AFindData)
{
	return lstrcmpi(AFindData.cFileName, ".") == 0;
}
//---------------------------------------------------------------------------
bool __fastcall IsDotDotFileName(const WIN32_FIND_DATA& AFindData)
{
	return lstrcmpi(AFindData.cFileName, "..") == 0;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Files
//---------------------------------------------------------------------------

