//---------------------------------------------------------------------------
// Forms.cpp
//
// Version: 1.1.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Forms.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Forms {
//---------------------------------------------------------------------------
void __fastcall TAppInstanceLock::Initialize()
{
	try {
		Unmark();
		Unlock();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TAppInstanceLock::Uninitialize()
{
	Unmark();
	Unlock();
}
//---------------------------------------------------------------------------
void __fastcall TAppInstanceLock::Lock()
{
	::SetLastError(NO_ERROR);
	HANDLE newHandle = CreateMutex(NULL, true, FName.c_str());
	if(newHandle == NULL) {
		throw EAppInstanceError("The program cannot lock the application instance.");
	}
	if(::GetLastError() != NO_ERROR) {
		CloseHandle(newHandle);
		throw EMultipleAppInstances("An application instance already exists.");
	}
	FHandle = newHandle;
	FLocked = true;
}
//---------------------------------------------------------------------------
void __fastcall TAppInstanceLock::Unlock()
{
	if(!FLocked) {
		return;
	}
	CloseHandle(FHandle);
	FHandle = NULL;
	FLocked = false;
}
//---------------------------------------------------------------------------
void __fastcall TAppInstanceLock::Mark()
{
	struct Local
	{
		static BOOL CALLBACK MatchName(HWND hwnd, LPTSTR lpszString, HANDLE hData, DWORD dwData)
		{
			if(lpszString == NULL) {
				return true;
			}
			if(HIWORD(reinterpret_cast<DWORD>(lpszString)) == 0) {
				// Ignore atoms
				return true;
			}
			return lstrcmp(lpszString, (*reinterpret_cast<AnsiString*>(dwData)).c_str()) != 0;
		}
	};

	if(EnumPropsEx(Application->Handle, reinterpret_cast<PROPENUMPROCEX>(Local::MatchName), reinterpret_cast<LPARAM>(&FName)) == static_cast<int>(false) ||
		!SetProp(Application->Handle, FName.c_str(), reinterpret_cast<HANDLE>(!NULL))) {
		throw EAppInstanceError("The program cannot mark the application instance.");
	}
	FMarked = true;
}
//---------------------------------------------------------------------------
void __fastcall TAppInstanceLock::Unmark()
{
	if(!FMarked) {
		return;
	}
	RemoveProp(Application->Handle, FName.c_str());
	FMarked = false;
}
//---------------------------------------------------------------------------
void __fastcall TAppInstanceLock::ActivateFirst(const AnsiString& AName)
{
	struct Local
	{
		static void __fastcall ActivateInstance(HWND AHandle, void* AData)
		{
			if(!IsWindowVisible(AHandle)) {
				return;
			}
			SetForegroundWindow(AHandle);
		}
	};

	FindFirst(AName, Local::ActivateInstance, NULL);
}
//---------------------------------------------------------------------------
bool __fastcall TAppInstanceLock::FindFirst(const AnsiString& AName, TFindFirstProc AProc, void* AData)
{
	struct Local
	{
		struct TFindFirstData
		{
			AnsiString Name;
			TFindFirstProc Proc;
			void* Data;
			bool Found;
		};
		static BOOL CALLBACK GetNext(HWND hwnd, LPARAM lParam)
		{
			TFindFirstData& data = *reinterpret_cast<TFindFirstData*>(lParam);
			if(GetProp(hwnd, data.Name.c_str()) == NULL) {
				return true;
			}
			data.Proc(hwnd, data.Data);
			data.Found = true;
			return false;
		}
	};

	Local::TFindFirstData data;
	data.Name = AName;
	data.Proc = AProc;
	data.Data = AData;
	data.Found = false;
	EnumWindows(reinterpret_cast<WNDENUMPROC>(Local::GetNext), reinterpret_cast<LPARAM>(&data));
	return data.Found;
}
//---------------------------------------------------------------------------
__fastcall TAppInstanceLock::TAppInstanceLock(const AnsiString& AName) :
	FHandle(NULL),
	FName(AName),
	FLocked(false),
	FMarked(false)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TAppInstanceLock::~TAppInstanceLock()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
TPoint __fastcall TFormBounds::GetMinPos()
{
	return GetPlacement().ptMinPosition;
}
//---------------------------------------------------------------------------
void __fastcall TFormBounds::SetMinPos(const TPoint& AMinPos)
{
	WINDOWPLACEMENT placement = GetPlacement();
	placement.ptMinPosition = AMinPos;
	SetPlacement(placement);
}
//---------------------------------------------------------------------------
TPoint __fastcall TFormBounds::GetMaxPos()
{
	return GetPlacement().ptMaxPosition;
}
//---------------------------------------------------------------------------
void __fastcall TFormBounds::SetMaxPos(const TPoint& AMaxPos)
{
	WINDOWPLACEMENT placement = GetPlacement();
	placement.ptMaxPosition = AMaxPos;
	SetPlacement(placement);
}
//---------------------------------------------------------------------------
TRect __fastcall TFormBounds::GetNormalBounds()
{
	return GetPlacement().rcNormalPosition;
}
//---------------------------------------------------------------------------
void __fastcall TFormBounds::SetNormalBounds(const TRect& ANormalBounds)
{
	WINDOWPLACEMENT placement = GetPlacement();
	placement.rcNormalPosition = ANormalBounds;
	SetPlacement(placement);
}
//---------------------------------------------------------------------------
WINDOWPLACEMENT __fastcall TFormBounds::GetPlacement()
{
	WINDOWPLACEMENT placement = {0};
	placement.length = sizeof(WINDOWPLACEMENT);
	if(!GetWindowPlacement(Handle, &placement)) {
		throw EFormError("The program cannot get the form placement.");
	}
	return placement;
}
//---------------------------------------------------------------------------
void __fastcall TFormBounds::SetPlacement(const WINDOWPLACEMENT& APlacement)
{
	WINDOWPLACEMENT placement = APlacement;
	placement.length = sizeof(WINDOWPLACEMENT);
	placement.flags = 0;
	placement.showCmd = GetPlacement().showCmd;
	if(!SetWindowPlacement(Handle, &placement)) {
		throw EFormError("The program cannot get the form placement.");
	}
}
//---------------------------------------------------------------------------
__fastcall TFormBounds::TFormBounds(TCustomForm* AForm) :
	FHandle(AForm->Handle)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFormBounds::TFormBounds(HWND AHandle) :
	FHandle(AHandle)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFormBounds::~TFormBounds()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TAutoScreenCursor::TAutoScreenCursor(TCursor ACursor) :
	FCursor(Screen->Cursor)
{
	Screen->Cursor = ACursor;
}
//---------------------------------------------------------------------------
__fastcall TAutoScreenCursor::~TAutoScreenCursor()
{
	Screen->Cursor = FCursor;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Forms
//---------------------------------------------------------------------------

