//---------------------------------------------------------------------------
#ifndef Moochos_Moa_Forms_H
#define Moochos_Moa_Forms_H
//---------------------------------------------------------------------------
// Forms.h
//
// Version: 1.1.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Forms {
//---------------------------------------------------------------------------
class EAppInstanceError : public Exception
{
public:
	__fastcall EAppInstanceError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class EMultipleAppInstances : public Exception
{
public:
	__fastcall EMultipleAppInstances(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TAppInstanceLock
	// Prevents multiple application instances. The name passed to the
	// constructor is internally used to create a mutex object with. If more
	// than one application instances are tried to start, throws
	// EMultipleAppInstances.
{
public:
	typedef void __fastcall (* TFindFirstProc)(HWND AHandle, void* AData);
private:
	HANDLE FHandle;
	AnsiString FName;
	bool FLocked;
	bool FMarked;
	__fastcall TAppInstanceLock(const TAppInstanceLock& AOther); // Forbidden
	TAppInstanceLock& __fastcall operator =(const TAppInstanceLock& AOther); // Forbidden
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall Lock();
	void __fastcall Unlock();
	void __fastcall Mark();
	void __fastcall Unmark();
protected:
public:
	static void __fastcall ActivateFirst(const AnsiString& AName);
	static bool __fastcall FindFirst(const AnsiString& AName, TFindFirstProc AFindFirstProc, void* AData);
	__fastcall TAppInstanceLock(const AnsiString& AName);
	__fastcall ~TAppInstanceLock();
};
//---------------------------------------------------------------------------
class EFormError : public Exception
{
public:
	__fastcall EFormError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TFormBounds
	// Represents the bounds of a form.
{
private:
	HWND FHandle;
	__fastcall TFormBounds(const TFormBounds& AOther); // Forbidden
	TFormBounds& __fastcall operator =(const TFormBounds& AOther); // Forbidden
	TPoint __fastcall GetMinPos();
	void __fastcall SetMinPos(const TPoint& AMinPos);
	TPoint __fastcall GetMaxPos();
	void __fastcall SetMaxPos(const TPoint& AMaxPos);
	TRect __fastcall GetNormalBounds();
	void __fastcall SetNormalBounds(const TRect& ANormalBounds);
	WINDOWPLACEMENT __fastcall GetPlacement();
	void __fastcall SetPlacement(const WINDOWPLACEMENT& APlacement);
protected:
public:
	__fastcall TFormBounds(TCustomForm* AForm);
	__fastcall TFormBounds(HWND AHandle);
	__fastcall ~TFormBounds();
	__property HWND Handle = {read=FHandle};
	__property TPoint MinPos = {read=GetMinPos, write=SetMinPos};
	__property TPoint MaxPos = {read=GetMaxPos, write=SetMaxPos};
	__property TRect NormalBounds = {read=GetNormalBounds, write=SetNormalBounds};
};
//---------------------------------------------------------------------------
class EScreenError : public Exception
{
public:
	__fastcall EScreenError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TAutoScreenCursor
	// Temporarily changes Cursor of the Screen until this object is freed.
{
private:
	TCursor FCursor;
	__fastcall TAutoScreenCursor(const TAutoScreenCursor& AOther); // Forbidden
	TAutoScreenCursor& __fastcall operator =(const TAutoScreenCursor& AOther); // Forbidden
protected:
public:
	__fastcall TAutoScreenCursor(TCursor ACursor);
	__fastcall ~TAutoScreenCursor();
};
//---------------------------------------------------------------------------
}}} // End of Moochos::Moa::Forms
using namespace Moochos::Moa::Forms;
//---------------------------------------------------------------------------
#endif

