//---------------------------------------------------------------------------
#ifndef Moochos_Moa_ListControls_H
#define Moochos_Moa_ListControls_H
//---------------------------------------------------------------------------
// ListControls.h
//
// Version: 1.1.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <ComCtrls.hpp>
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace ListControls {
//---------------------------------------------------------------------------
// List control shells
//---------------------------------------------------------------------------
class EListControlShellError : public Exception
{
public:
	__fastcall EListControlShellError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
extern const int lsCut;
extern const int lsDropTarget;
extern const int lsFocused;
extern const int lsSelected;
//---------------------------------------------------------------------------
class TListControlShellView;
class TListControlShell : public TObject
	// Provides functions to edit items of a list control that is working in
	// a virtual mode.
{
public:
	typedef int __fastcall (* TSortCompareProc)(int AIndex1, int AIndex2, void* AData);
private:
	struct TSortItem
	{
		int Index;
		TSortCompareProc CompareProc;
		void* CompareData;
	};
	TListControlShellView* FView;
	TIntegerDynArray FStates;
	int FCount;
	int FFocusIndex;
	int FSelCount;
	static int __fastcall SortListCompare(void* AItem1, void* AItem2);
	void __fastcall Expand();
	int __fastcall GetState(int AIndex);
	void __fastcall SetState(int AIndex, int AState);
	void __fastcall SetCount(int ACount);
	int __fastcall GetCapacity();
	void __fastcall SetCapacity(int ACapacity);
	void __fastcall SetFocusIndex(int AFocusIndex);
	TBooleanDynArray __fastcall GetSelTable();
	void __fastcall SetSelTable(const TBooleanDynArray& ASelTable);
	TIntegerDynArray __fastcall MakeOrder(bool AAscending);
protected:
public:
	__fastcall TListControlShell(TListControlShellView* AView);
	virtual __fastcall ~TListControlShell();
	int __fastcall Add();
	void __fastcall Insert(int AIndex);
	void __fastcall Delete(int AIndex);
	void __fastcall DeleteSelected();
	void __fastcall DeleteItems(const TBooleanDynArray& ACanDelete);
	void __fastcall Clear();
	void __fastcall SelectAll();
	void __fastcall ClearSelection();
	void __fastcall ReverseSelection();
	void __fastcall SelectItems(const TBooleanDynArray& ACanSelect);
	void __fastcall Exchange(int AIndex1, int AIndex2);
	void __fastcall Move(int AOldIndex, int ANewIndex);
	TIntegerDynArray __fastcall MoveSelected(int ADistance);
	TIntegerDynArray __fastcall Sort(TSortCompareProc ACompareProc, void* ACompareData);
	TIntegerDynArray __fastcall Invert();
	void __fastcall Reorder(const TIntegerDynArray& AOrder);
	void __fastcall Update();
	void __fastcall UpdateControl();
	__property TListControlShellView* View = {read=FView};
	__property int States[int AIndex] = {read=GetState, write=SetState};
	__property int Count = {read=FCount, write=SetCount};
	__property int Capacity = {read=GetCapacity, write=SetCapacity};
	__property int FocusIndex = {read=FFocusIndex, write=SetFocusIndex};
	__property int SelCount = {read=FSelCount};
	__property TBooleanDynArray SelTable = {read=GetSelTable, write=SetSelTable};
};
//---------------------------------------------------------------------------
class TListControlShellView : public TObject
	// An abstract list control for TListControlShell.
{
private:
	TWinControl* FControl;
protected:
	virtual int __fastcall GetState(int AIndex) = 0;
	virtual void __fastcall SetState(int AIndex, int AState) = 0;
	virtual int __fastcall GetCount() = 0;
	virtual void __fastcall SetCount(int ACount) = 0;
	virtual int __fastcall GetFocusIndex() = 0;
	virtual void __fastcall SetFocusIndex(int AFocusIndex) = 0;
	virtual int __fastcall GetSelCount() = 0;
public:
	__fastcall TListControlShellView(TWinControl* AControl);
	virtual __fastcall ~TListControlShellView();
	virtual void __fastcall BeginUpdate() = 0;
	virtual void __fastcall EndUpdate() = 0;
	__property TWinControl* Control = {read=FControl};
	__property int States[int AIndex] = {read=GetState, write=SetState};
	__property int Count = {read=GetCount, write=SetCount};
	__property int FocusIndex = {read=GetFocusIndex, write=SetFocusIndex};
	__property int SelCount = {read=GetSelCount};
};
//---------------------------------------------------------------------------
class TListViewShellView : public TListControlShellView
	// Wraps TListView.
{
private:
protected:
	virtual TListView* __fastcall GetControl();
	virtual int __fastcall GetState(int AIndex);
	virtual void __fastcall SetState(int AIndex, int AState);
	virtual int __fastcall GetCount();
	virtual void __fastcall SetCount(int ACount);
	virtual int __fastcall GetFocusIndex();
	virtual void __fastcall SetFocusIndex(int AFocusIndex);
	virtual int __fastcall GetSelCount();
public:
	__fastcall TListViewShellView(TListView* AControl);
	virtual __fastcall ~TListViewShellView();
	virtual void __fastcall BeginUpdate();
	virtual void __fastcall EndUpdate();
	__property TListView* Control = {read=GetControl};
};
//---------------------------------------------------------------------------
class TListBoxShellView : public TListControlShellView
	// Wraps TListBox.
{
private:
protected:
	virtual TListBox* __fastcall GetControl();
	virtual int __fastcall GetState(int AIndex);
	virtual void __fastcall SetState(int AIndex, int AState);
	virtual int __fastcall GetCount();
	virtual void __fastcall SetCount(int ACount);
	virtual int __fastcall GetFocusIndex();
	virtual void __fastcall SetFocusIndex(int AFocusIndex);
	virtual int __fastcall GetSelCount();
public:
	__fastcall TListBoxShellView(TListBox* AControl);
	virtual __fastcall ~TListBoxShellView();
	virtual void __fastcall BeginUpdate();
	virtual void __fastcall EndUpdate();
	__property TListBox* Control = {read=GetControl};
};
//---------------------------------------------------------------------------
// List control drawing
//---------------------------------------------------------------------------
class TListItemLayout : public TObject
	// Calculates the bounds of list items.
{
private:
	int FHorzPadding;
	int FVertPadding;
	int FIconWidth;
	int FIconHeight;
	int FTextHeight;
	int __fastcall GetItemHeight();
protected:
public:
	__fastcall TListItemLayout();
	virtual __fastcall ~TListItemLayout();
	TRect __fastcall CalculateIconBounds(const TRect& ABounds);
	TRect __fastcall CalculateTextBounds(const TRect& ABounds);
	__property int HorzPadding = {read=FHorzPadding, write=FHorzPadding};
	__property int VertPadding = {read=FVertPadding, write=FVertPadding};
	__property int IconWidth = {read=FIconWidth, write=FIconWidth};
	__property int IconHeight = {read=FIconHeight, write=FIconHeight};
	__property int TextHeight = {read=FTextHeight, write=FTextHeight};
	__property int ItemHeight = {read=GetItemHeight};
};
//---------------------------------------------------------------------------
// List box utilities
//---------------------------------------------------------------------------
extern void __fastcall SelectItems(TListBox* AControl, int AFirstIndex, int ALastIndex, bool ASelect);
extern void __fastcall UpdateItems(TListBox* AControl, int AFirstIndex, int ALastIndex);
extern void __fastcall MakeItemVisible(TListBox* AControl, int AIndex);
extern int __fastcall IncrementalFindCaption(TListBox* AControl, const AnsiString& AFindText);
//---------------------------------------------------------------------------
// List view utilities
//---------------------------------------------------------------------------
class TAutoListViewOnData
	// Temporarily changes OnData of a list view until this object is freed.
	// For example, to gain better performance, set OnData to NULL while
	// editing a list view.
{
private:
	TListView* FListView;
	TLVOwnerDataEvent FOnData;
	__fastcall TAutoListViewOnData(const TAutoListViewOnData& AOther); // Forbidden
	TAutoListViewOnData& __fastcall operator =(const TAutoListViewOnData& AOther); // Forbidden
protected:
public:
	__fastcall TAutoListViewOnData(TListView* AListView, TLVOwnerDataEvent AOnData = NULL);
	__fastcall ~TAutoListViewOnData();
};
//---------------------------------------------------------------------------
class TAutoListItemsUpdator
	// Calls BeginUpdate on construction and EndUpdate on destruction.
{
private:
	TListItems* FListItems;
	__fastcall TAutoListItemsUpdator(const TAutoListItemsUpdator& AOther); // Forbidden
	TAutoListItemsUpdator& __fastcall operator =(const TAutoListItemsUpdator& AOther); // Forbidden
protected:
public:
	__fastcall TAutoListItemsUpdator(TListItems* AListItems);
	__fastcall ~TAutoListItemsUpdator();
};
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::ListControls
using namespace Moochos::Moa::ListControls;
//---------------------------------------------------------------------------
#endif

