//---------------------------------------------------------------------------
// Shell.cpp
//
// Version: 1.1.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "Shell.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Shell {
//---------------------------------------------------------------------------
void __fastcall TFileDrop::EnsureNotFinished()
{
	if(Finished) {
		throw EShellError("The file drop operation is already finished.");
	}
}
//---------------------------------------------------------------------------
HDROP __fastcall TFileDrop::GetHandle()
{
	EnsureNotFinished();
	return reinterpret_cast<HDROP>(FMessage.WParam);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileDrop::GetFile(int AIndex)
{
	EnsureNotFinished();
	if(AIndex < 0 || AIndex >= Count) {
		throw EShellError("The index for a dropped file is out of the range.");
	}
	UINT toCopySize = DragQueryFile(Handle, AIndex, NULL, 0); // Not including the terminator
	const UINT bufferSize = MAX_PATH;
	TCHAR buffer[bufferSize] = {0};
	UINT didCopySize = DragQueryFile(Handle, AIndex, buffer, bufferSize); // Not including the terminator
	if(didCopySize != toCopySize) {
		throw EShellError("The program cannot get the path of a dropped file.");
	}
	return AnsiString(buffer, didCopySize);
}
//---------------------------------------------------------------------------
int __fastcall TFileDrop::GetCount()
{
	EnsureNotFinished();
	return DragQueryFile(Handle, 0xFFFFFFFF, NULL, 0);
}
//---------------------------------------------------------------------------
TPoint __fastcall TFileDrop::GetPoint()
{
	EnsureNotFinished();
	TPoint point;
	DragQueryPoint(Handle, &point);
	return point;
}
//---------------------------------------------------------------------------
__fastcall TFileDrop::TFileDrop(const TMessage& AMessage, bool AAutoFinish) :
	FMessage(),
	FAutoFinish(AAutoFinish),
	FFinished(false)
{
	if(AMessage.Msg != WM_DROPFILES) {
		throw EShellError("The message is not WM_DROPFILES.");
	}
	FMessage = AMessage;
}
//---------------------------------------------------------------------------
__fastcall TFileDrop::~TFileDrop()
{
	if(AutoFinish) {
		Finish();
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileDrop::GetFiles(TStrings* AFiles)
{
	TPtr<TStrings> files = new TStringList();
	files->Capacity = Count;
	for(int i = 0; i < Count; i++) {
		files->Add(Files[i]);
	}
	AFiles->Assign(files);
}
//---------------------------------------------------------------------------
void __fastcall TFileDrop::Finish()
{
	if(Finished) {
		return;
	}
	DragFinish(Handle);
	FFinished = true;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::Initialize()
{
	try {
		InitializeInterfaces();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::Uninitialize()
{
	UninitializeInterfaces();
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::InitializeInterfaces()
{
	static const char* errMsg = "The program cannot initialize the interface of a shortcut file.";

	CoInitialize(NULL);
	if(FAILED(CoCreateInstance(
		CLSID_ShellLink,
		NULL,
		CLSCTX_INPROC_SERVER,
		IID_IShellLink,
		reinterpret_cast<void**>(&FShellLink)))) {
		throw EShellLinkError(errMsg);
	}
	if(FAILED(ShellLink->QueryInterface(
		IID_IPersistFile,
		reinterpret_cast<void**>(&FPersistFile)))) {
		throw EShellLinkError(errMsg);
	}
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::UninitializeInterfaces()
{
	try {
		if(FPersistFile != NULL) {
			FPersistFile->Release();
			FPersistFile = NULL;
		}
		if(FShellLink != NULL) {
			FShellLink->Release();
			FShellLink = NULL;
		}
	}
	__finally {
		CoUninitialize();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShellLink::GetTargetFile()
{
	const int bufferSize = MAX_PATH;
	CHAR buffer[bufferSize] = {0};
	WIN32_FIND_DATA findData = {0};
	if(FAILED(ShellLink->GetPath(buffer, bufferSize, &findData, SLGP_UNCPRIORITY))) {
		GetInError();
	}
	return AnsiString(buffer);
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetTargetFile(const AnsiString& ATargetFile)
{
	if(FAILED(ShellLink->SetPath(ATargetFile.c_str()))) {
		SetInError();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShellLink::GetWorkingDir()
{
	const int bufferSize = MAX_PATH;
	CHAR buffer[bufferSize] = {0};
	if(FAILED(ShellLink->GetWorkingDirectory(buffer, bufferSize))) {
		GetInError();
	}
	return AnsiString(buffer);
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetWorkingDir(const AnsiString& AWorkingDir)
{
	if(FAILED(ShellLink->SetWorkingDirectory(AWorkingDir.c_str()))) {
		SetInError();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShellLink::GetArguments()
{
	const int bufferSize = MAX_PATH;
	CHAR buffer[bufferSize] = {0};
	if(FAILED(ShellLink->GetArguments(buffer, bufferSize))) {
		GetInError();
	}
	return AnsiString(buffer);
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetArguments(const AnsiString& AArguments)
{
	if(FAILED(ShellLink->SetArguments(AArguments.c_str()))) {
		SetInError();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShellLink::GetDescription()
{
	const int bufferSize = MAX_PATH;
	CHAR buffer[bufferSize] = {0};
	if(FAILED(ShellLink->GetDescription(buffer, bufferSize))) {
		GetInError();
	}
	return AnsiString(buffer);
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetDescription(const AnsiString& ADescription)
{
	if(FAILED(ShellLink->SetDescription(ADescription.c_str()))) {
		SetInError();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShellLink::GetIconFile()
{
	AnsiString iconFile;
	int iconIndex;
	GetIconLocation(iconFile, iconIndex);
	return iconFile;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetIconFile(const AnsiString& AIconFile)
{
	SetIconLocation(AIconFile, IconIndex);
}
//---------------------------------------------------------------------------
int __fastcall TShellLink::GetIconIndex()
{
	AnsiString iconFile;
	int iconIndex;
	GetIconLocation(iconFile, iconIndex);
	return iconIndex;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetIconIndex(int AIconIndex)
{
	SetIconLocation(IconFile, AIconIndex);
}
//---------------------------------------------------------------------------
int __fastcall TShellLink::GetShowCommand()
{
	int showCommand;
	if(FAILED(ShellLink->GetShowCmd(&showCommand))) {
		GetInError();
	}
	return showCommand;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetShowCommand(int AShowCommand)
{
	if(FAILED(ShellLink->SetShowCmd(AShowCommand))) {
		SetInError();
	}
}
//---------------------------------------------------------------------------
WORD __fastcall TShellLink::GetHotKey()
{
	WORD hotKey;
	if(FAILED(ShellLink->GetHotkey(&hotKey))) {
		GetInError();
	}
	return hotKey;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetHotKey(WORD AHotKey)
{
	if(FAILED(ShellLink->SetHotkey(AHotKey))) {
		SetInError();
	}
}
//---------------------------------------------------------------------------
LPITEMIDLIST __fastcall TShellLink::GetIdList()
{
	LPITEMIDLIST idList = NULL;
	if(FAILED(ShellLink->GetIDList(&idList))) {
		GetInError();
	}
	return idList;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetIdList(LPCITEMIDLIST AIdList)
{
	if(FAILED(ShellLink->SetIDList(AIdList))) {
		SetInError();
	}
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::GetInError()
{
	throw EShellLinkError("The program cannot get a property of the shortcut file.");
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetInError()
{
	throw EShellLinkError("The program cannot set a property of the shortcut file.");
}
//---------------------------------------------------------------------------
__fastcall TShellLink::TShellLink() :
	TObject(),
	FShellLink(NULL),
	FPersistFile(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TShellLink::~TShellLink()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::LoadFromFile(const AnsiString& AFileName)
{
	if(FAILED(PersistFile->Load(WideString(AFileName), STGM_READ | STGM_SHARE_DENY_WRITE))) {
		throw EShellLinkError("The program cannot load the shortcut file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SaveToFile(const AnsiString& AFileName)
{
	if(FAILED(PersistFile->Save(WideString(AFileName), true))) {
		throw EShellLinkError("The program cannot save the shortcut file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::Resolve(HWND AParent, DWORD AFlags)
{
	if(FAILED(ShellLink->Resolve(AParent, AFlags))) {
		throw EShellLinkError("The program cannot resolve the shortcut file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::GetIconLocation(AnsiString& AIconFile, int& AIconIndex)
{
	const int bufferSize = MAX_PATH;
	CHAR buffer[bufferSize] = {0};
	int iconIndex;
	if(FAILED(ShellLink->GetIconLocation(buffer, bufferSize, &AIconIndex))) {
		GetInError();
	}
	AIconFile = buffer;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetIconLocation(const AnsiString& AIconFile, int AIconIndex)
{
	if(FAILED(ShellLink->SetIconLocation(AIconFile.c_str(), AIconIndex))) {
		SetInError();
	}
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Shell
//---------------------------------------------------------------------------

