//---------------------------------------------------------------------------
// Sync.cpp
//
// Version: 1.1.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Sync.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Sync {
//---------------------------------------------------------------------------
__fastcall TMutex::TMutex(const AnsiString& AName, bool ATryLockNow) :
	FHandle(NULL),
	FLocked(false)
{
	::SetLastError(NO_ERROR);
	HANDLE newHandle = CreateMutex(NULL, ATryLockNow, AName.c_str());
	if(newHandle == NULL) {
		throw ELockError("The program cannot create a mutex object.");
	}
	FHandle = newHandle;
	if(ATryLockNow) {
		if(::GetLastError() != ERROR_ALREADY_EXISTS) {
			FLocked = true;
		}
	}
}
//---------------------------------------------------------------------------
__fastcall TMutex::~TMutex()
{
	Unlock();

	CloseHandle(FHandle);
	FHandle = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TMutex::Lock(int ATimeoutInterval)
{
	if(!TryLock(ATimeoutInterval)) {
		throw ELockFailure("The program cannot get the ownership of a mutex object.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TMutex::TryLock(int ATimeoutInterval)
{
	if(IsLocked()) {
		return true;
	}
	switch(WaitForSingleObject(Handle, ATimeoutInterval < 0 ? INFINITE : ATimeoutInterval)) {
	case WAIT_ABANDONED:
	case WAIT_OBJECT_0:
		FLocked = true;
		return true;
	case WAIT_TIMEOUT:
	default:
		return false;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMutex::Unlock()
{
	if(!IsLocked()) {
		return;
	}
	ReleaseMutex(FHandle);
	FLocked = false;
}
//---------------------------------------------------------------------------
bool __fastcall TMutex::IsLocked()
{
	return FLocked;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Sync
//---------------------------------------------------------------------------

